@@ -32,14 +32,9 @@ def example_tag():
     return tags.Tag("py3", "none", "any")
 
 
-@pytest.fixture
-def is_x86():
-    return re.match(r"(i\d86|x86_64)", platform.machine()) is not None
-
-
 @pytest.fixture
 def manylinux_module(monkeypatch):
-    monkeypatch.setattr(tags, "_have_compatible_glibc", lambda *args: False)
+    monkeypatch.setattr(tags, "_get_glibc_version", lambda *args: (2, 20))
     module_name = "_manylinux"
     module = types.ModuleType(module_name)
     monkeypatch.setitem(sys.modules, module_name, module)
@@ -195,9 +190,9 @@ class TestInterpreterVersion:
         "version_info,version_str",
         [
             ((1, 2, 3), "12"),
-            ((1, 12, 3), "1_12"),
-            ((11, 2, 3), "11_2"),
-            ((11, 12, 3), "11_12"),
+            ((1, 12, 3), "112"),
+            ((11, 2, 3), "112"),
+            ((11, 12, 3), "1112"),
             ((1, 2, 13), "12"),
         ],
     )
@@ -225,21 +220,40 @@ class TestMacOSPlatforms:
     @pytest.mark.parametrize(
         "version,arch,expected",
         [
-            ((10, 17), "x86_64", ["x86_64", "intel", "fat64", "fat32", "universal"]),
-            ((10, 4), "x86_64", ["x86_64", "intel", "fat64", "fat32", "universal"]),
+            (
+                (10, 15),
+                "x86_64",
+                ["x86_64", "intel", "fat64", "fat32", "universal2", "universal"],
+            ),
+            (
+                (10, 4),
+                "x86_64",
+                ["x86_64", "intel", "fat64", "fat32", "universal2", "universal"],
+            ),
             ((10, 3), "x86_64", []),
-            ((10, 17), "i386", ["i386", "intel", "fat32", "fat", "universal"]),
+            ((10, 15), "i386", ["i386", "intel", "fat32", "fat", "universal"]),
             ((10, 4), "i386", ["i386", "intel", "fat32", "fat", "universal"]),
+            ((10, 3), "intel", ["intel", "universal"]),
+            ((10, 5), "intel", ["intel", "universal"]),
+            ((10, 15), "intel", ["intel", "universal"]),
             ((10, 3), "i386", []),
-            ((10, 17), "ppc64", []),
+            ((10, 15), "ppc64", []),
             ((10, 6), "ppc64", []),
             ((10, 5), "ppc64", ["ppc64", "fat64", "universal"]),
             ((10, 3), "ppc64", []),
-            ((10, 17), "ppc", []),
+            ((10, 15), "ppc", []),
             ((10, 7), "ppc", []),
             ((10, 6), "ppc", ["ppc", "fat32", "fat", "universal"]),
             ((10, 0), "ppc", ["ppc", "fat32", "fat", "universal"]),
-            ((11, 0), "riscv", ["riscv", "universal"]),
+            ((11, 0), "riscv", ["riscv"]),
+            (
+                (11, 0),
+                "x86_64",
+                ["x86_64", "intel", "fat64", "fat32", "universal2", "universal"],
+            ),
+            ((11, 0), "arm64", ["arm64", "universal2"]),
+            ((11, 1), "arm64", ["arm64", "universal2"]),
+            ((12, 0), "arm64", ["arm64", "universal2"]),
         ],
     )
     def test_binary_formats(self, version, arch, expected):
@@ -271,40 +285,79 @@ class TestMacOSPlatforms:
             "macosx_10_5_intel",
             "macosx_10_5_fat64",
             "macosx_10_5_fat32",
+            "macosx_10_5_universal2",
             "macosx_10_5_universal",
             "macosx_10_4_x86_64",
             "macosx_10_4_intel",
             "macosx_10_4_fat64",
             "macosx_10_4_fat32",
+            "macosx_10_4_universal2",
             "macosx_10_4_universal",
         ]
 
-        assert len(list(tags.mac_platforms((10, 17), "x86_64"))) == 14 * 5
+        assert len(list(tags.mac_platforms((10, 17), "x86_64"))) == 14 * 6
 
         assert not list(tags.mac_platforms((10, 0), "x86_64"))
 
+    @pytest.mark.parametrize("major,minor", [(11, 0), (11, 3), (12, 0), (12, 3)])
+    def test_macos_11(self, major, minor):
+        platforms = list(tags.mac_platforms((major, minor), "x86_64"))
+        assert "macosx_11_0_arm64" not in platforms
+        assert "macosx_11_0_x86_64" in platforms
+        assert "macosx_11_3_x86_64" not in platforms
+        assert "macosx_11_0_universal" in platforms
+        assert "macosx_11_0_universal2" in platforms
+        # Mac OS "10.16" is the version number that binaries compiled against an old
+        # (pre 11.0) SDK will see.   It can also be enabled explicitly for a process
+        # with the environment variable SYSTEM_VERSION_COMPAT=1.
+        assert "macosx_10_16_x86_64" in platforms
+        assert "macosx_10_15_x86_64" in platforms
+        assert "macosx_10_4_x86_64" in platforms
+        assert "macosx_10_3_x86_64" not in platforms
+        if major >= 12:
+            assert "macosx_12_0_x86_64" in platforms
+            assert "macosx_12_0_universal" in platforms
+            assert "macosx_12_0_universal2" in platforms
+
+        platforms = list(tags.mac_platforms((major, minor), "arm64"))
+        assert "macosx_11_0_arm64" in platforms
+        assert "macosx_11_3_arm64" not in platforms
+        assert "macosx_11_0_universal" not in platforms
+        assert "macosx_11_0_universal2" in platforms
+        assert "macosx_10_15_x86_64" not in platforms
+        assert "macosx_10_4_x86_64" not in platforms
+        assert "macosx_10_3_x86_64" not in platforms
+        if major >= 12:
+            assert "macosx_12_0_arm64" in platforms
+            assert "macosx_12_0_universal2" in platforms
+
 
 class TestManylinuxPlatform:
-    def test_module_declaration_true(self, manylinux_module):
-        manylinux_module.manylinux1_compatible = True
-        assert tags._is_manylinux_compatible("manylinux1", (2, 5))
+    def teardown_method(self):
+        # Clear the version cache
+        tags._glibc_version = []
 
-    def test_module_declaration_false(self, manylinux_module):
-        manylinux_module.manylinux1_compatible = False
-        assert not tags._is_manylinux_compatible("manylinux1", (2, 5))
-
-    def test_module_declaration_missing_attribute(self, manylinux_module):
-        try:
-            del manylinux_module.manylinux1_compatible
-        except AttributeError:
-            pass
-        assert not tags._is_manylinux_compatible("manylinux1", (2, 5))
-
-    def test_is_manylinux_compatible_module_support(
-        self, manylinux_module, monkeypatch
+    @pytest.mark.parametrize("tf", (True, False))
+    @pytest.mark.parametrize(
+        "attribute,glibc", (("1", (2, 5)), ("2010", (2, 12)), ("2014", (2, 17)))
+    )
+    def test_module_declaration(
+        self, monkeypatch, manylinux_module, attribute, glibc, tf
     ):
-        monkeypatch.setitem(sys.modules, manylinux_module.__name__, None)
-        assert not tags._is_manylinux_compatible("manylinux1", (2, 5))
+        manylinux = "manylinux{}_compatible".format(attribute)
+        monkeypatch.setattr(manylinux_module, manylinux, tf, raising=False)
+        res = tags._is_manylinux_compatible(manylinux, "x86_64", glibc)
+        assert tf is res
+
+    @pytest.mark.parametrize(
+        "attribute,glibc", (("1", (2, 5)), ("2010", (2, 12)), ("2014", (2, 17)))
+    )
+    def test_module_declaration_missing_attribute(
+        self, monkeypatch, manylinux_module, attribute, glibc
+    ):
+        manylinux = "manylinux{}_compatible".format(attribute)
+        monkeypatch.delattr(manylinux_module, manylinux, raising=False)
+        assert tags._is_manylinux_compatible(manylinux, "x86_64", glibc)
 
     @pytest.mark.parametrize(
         "version,compatible", (((2, 0), True), ((2, 5), True), ((2, 10), False))
@@ -313,29 +366,16 @@ class TestManylinuxPlatform:
         self, version, compatible, monkeypatch
     ):
         monkeypatch.setitem(sys.modules, "_manylinux", None)
-        monkeypatch.setattr(
-            tags,
-            "_have_compatible_glibc",
-            lambda major, minor: (major, minor) <= (2, 5),
+        monkeypatch.setattr(tags, "_get_glibc_version", lambda: (2, 5))
+        assert (
+            bool(tags._is_manylinux_compatible("manylinux1", "any", version))
+            == compatible
         )
-        assert bool(tags._is_manylinux_compatible("manylinux1", version)) == compatible
-
-    @pytest.mark.parametrize(
-        "version_str,major,minor,expected",
-        [
-            ("2.4", 2, 4, True),
-            ("2.4", 2, 5, False),
-            ("2.4", 2, 3, True),
-            ("3.4", 2, 4, False),
-        ],
-    )
-    def test_check_glibc_version(self, version_str, major, minor, expected):
-        assert expected == tags._check_glibc_version(version_str, major, minor)
 
     @pytest.mark.parametrize("version_str", ["glibc-2.4.5", "2"])
     def test_check_glibc_version_warning(self, version_str):
         with warnings.catch_warnings(record=True) as w:
-            tags._check_glibc_version(version_str, 2, 4)
+            tags._parse_glibc_version(version_str)
             assert len(w) == 1
             assert issubclass(w[0].category, RuntimeWarning)
 
@@ -373,6 +413,12 @@ class TestManylinuxPlatform:
         monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.20", raising=False)
         assert tags._glibc_version_string_confstr() == "2.20"
 
+    def test_glibc_version_string_fail(self, monkeypatch):
+        monkeypatch.setattr(os, "confstr", lambda x: None, raising=False)
+        monkeypatch.setitem(sys.modules, "ctypes", None)
+        assert tags._glibc_version_string() is None
+        assert tags._get_glibc_version() == (-1, -1)
+
     @pytest.mark.parametrize(
         "failure",
         [pretend.raiser(ValueError), pretend.raiser(OSError), lambda x: "XXX"],
@@ -389,6 +435,13 @@ class TestManylinuxPlatform:
         monkeypatch.setitem(sys.modules, "ctypes", None)
         assert tags._glibc_version_string_ctypes() is None
 
+    def test_glibc_version_string_ctypes_raise_oserror(self, monkeypatch):
+        def patched_cdll(name):
+            raise OSError("Dynamic loading not supported")
+
+        monkeypatch.setattr(ctypes, "CDLL", patched_cdll)
+        assert tags._glibc_version_string_ctypes() is None
+
     def test_get_config_var_does_not_log(self, monkeypatch):
         debug = pretend.call_recorder(lambda *a: None)
         monkeypatch.setattr(tags.logger, "debug", debug)
@@ -407,18 +460,18 @@ class TestManylinuxPlatform:
         ]
 
     @pytest.mark.skipif(platform.system() != "Linux", reason="requires Linux")
-    def test_have_compatible_glibc_linux(self):
+    def test_is_manylinux_compatible_old(self):
         # Assuming no one is running this test with a version of glibc released in
         # 1997.
-        assert tags._have_compatible_glibc(2, 0)
+        assert tags._is_manylinux_compatible("any", "any", (2, 0))
 
-    def test_have_compatible_glibc(self, monkeypatch):
+    def test_is_manylinux_compatible(self, monkeypatch):
         monkeypatch.setattr(tags, "_glibc_version_string", lambda: "2.4")
-        assert tags._have_compatible_glibc(2, 4)
+        assert tags._is_manylinux_compatible("", "any", (2, 4))
 
     def test_glibc_version_string_none(self, monkeypatch):
         monkeypatch.setattr(tags, "_glibc_version_string", lambda: None)
-        assert not tags._have_compatible_glibc(2, 4)
+        assert not tags._is_manylinux_compatible("any", "any", (2, 4))
 
     @pytest.mark.parametrize(
         "arch,is_32bit,expected",
@@ -433,59 +486,83 @@ class TestManylinuxPlatform:
         self, arch, is_32bit, expected, monkeypatch
     ):
         monkeypatch.setattr(distutils.util, "get_platform", lambda: arch)
+        monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.20", raising=False)
         monkeypatch.setattr(tags, "_is_manylinux_compatible", lambda *args: False)
         linux_platform = list(tags._linux_platforms(is_32bit=is_32bit))[-1]
         assert linux_platform == expected
 
     def test_linux_platforms_manylinux_unsupported(self, monkeypatch):
         monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
+        monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.20", raising=False)
         monkeypatch.setattr(tags, "_is_manylinux_compatible", lambda *args: False)
         linux_platform = list(tags._linux_platforms(is_32bit=False))
         assert linux_platform == ["linux_x86_64"]
 
-    def test_linux_platforms_manylinux1(self, is_x86, monkeypatch):
+    def test_linux_platforms_manylinux1(self, monkeypatch):
         monkeypatch.setattr(
-            tags, "_is_manylinux_compatible", lambda name, _: name == "manylinux1"
+            tags, "_is_manylinux_compatible", lambda name, *args: name == "manylinux1"
         )
-        if platform.system() != "Linux" or not is_x86:
-            monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
-            monkeypatch.setattr(platform, "machine", lambda: "x86_64")
+        monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
+        monkeypatch.setattr(platform, "machine", lambda: "x86_64")
+        monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.20", raising=False)
         platforms = list(tags._linux_platforms(is_32bit=False))
         arch = platform.machine()
         assert platforms == ["manylinux1_" + arch, "linux_" + arch]
 
-    def test_linux_platforms_manylinux2010(self, is_x86, monkeypatch):
-        monkeypatch.setattr(
-            tags, "_is_manylinux_compatible", lambda name, _: name == "manylinux2010"
-        )
-        if platform.system() != "Linux" or not is_x86:
-            monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
-            monkeypatch.setattr(platform, "machine", lambda: "x86_64")
-        platforms = list(tags._linux_platforms(is_32bit=False))
-        arch = platform.machine()
-        expected = ["manylinux2010_" + arch, "manylinux1_" + arch, "linux_" + arch]
-        assert platforms == expected
-
-    def test_linux_platforms_manylinux2014(self, is_x86, monkeypatch):
-        monkeypatch.setattr(
-            tags, "_is_manylinux_compatible", lambda name, _: name == "manylinux2014"
-        )
-        if platform.system() != "Linux" or not is_x86:
-            monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
-            monkeypatch.setattr(platform, "machine", lambda: "x86_64")
+    def test_linux_platforms_manylinux2010(self, monkeypatch):
+        monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
+        monkeypatch.setattr(platform, "machine", lambda: "x86_64")
+        monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.12", raising=False)
         platforms = list(tags._linux_platforms(is_32bit=False))
         arch = platform.machine()
         expected = [
-            "manylinux2014_" + arch,
+            "manylinux_2_12_" + arch,
             "manylinux2010_" + arch,
+            "manylinux_2_11_" + arch,
+            "manylinux_2_10_" + arch,
+            "manylinux_2_9_" + arch,
+            "manylinux_2_8_" + arch,
+            "manylinux_2_7_" + arch,
+            "manylinux_2_6_" + arch,
+            "manylinux_2_5_" + arch,
+            "manylinux1_" + arch,
+            "linux_" + arch,
+        ]
+        assert platforms == expected
+
+    def test_linux_platforms_manylinux2014(self, monkeypatch):
+        monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
+        monkeypatch.setattr(platform, "machine", lambda: "x86_64")
+        monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.17", raising=False)
+        platforms = list(tags._linux_platforms(is_32bit=False))
+        arch = platform.machine()
+        expected = [
+            "manylinux_2_17_" + arch,
+            "manylinux2014_" + arch,
+            "manylinux_2_16_" + arch,
+            "manylinux_2_15_" + arch,
+            "manylinux_2_14_" + arch,
+            "manylinux_2_13_" + arch,
+            "manylinux_2_12_" + arch,
+            "manylinux2010_" + arch,
+            "manylinux_2_11_" + arch,
+            "manylinux_2_10_" + arch,
+            "manylinux_2_9_" + arch,
+            "manylinux_2_8_" + arch,
+            "manylinux_2_7_" + arch,
+            "manylinux_2_6_" + arch,
+            "manylinux_2_5_" + arch,
             "manylinux1_" + arch,
             "linux_" + arch,
         ]
         assert platforms == expected
 
     def test_linux_platforms_manylinux2014_armhf_abi(self, monkeypatch):
+        monkeypatch.setattr(tags, "_glibc_version_string", lambda: "2.30")
         monkeypatch.setattr(
-            tags, "_is_manylinux_compatible", lambda name, _: name == "manylinux2014"
+            tags,
+            "_is_manylinux_compatible",
+            lambda name, *args: name == "manylinux2014",
         )
         monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_armv7l")
         monkeypatch.setattr(
@@ -498,9 +575,7 @@ class TestManylinuxPlatform:
         assert platforms == expected
 
     def test_linux_platforms_manylinux2014_i386_abi(self, monkeypatch):
-        monkeypatch.setattr(
-            tags, "_is_manylinux_compatible", lambda name, _: name == "manylinux2014"
-        )
+        monkeypatch.setattr(tags, "_glibc_version_string", lambda: "2.17")
         monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
         monkeypatch.setattr(
             sys,
@@ -509,18 +584,50 @@ class TestManylinuxPlatform:
         )
         platforms = list(tags._linux_platforms(is_32bit=True))
         expected = [
+            "manylinux_2_17_i686",
             "manylinux2014_i686",
+            "manylinux_2_16_i686",
+            "manylinux_2_15_i686",
+            "manylinux_2_14_i686",
+            "manylinux_2_13_i686",
+            "manylinux_2_12_i686",
             "manylinux2010_i686",
+            "manylinux_2_11_i686",
+            "manylinux_2_10_i686",
+            "manylinux_2_9_i686",
+            "manylinux_2_8_i686",
+            "manylinux_2_7_i686",
+            "manylinux_2_6_i686",
+            "manylinux_2_5_i686",
             "manylinux1_i686",
             "linux_i686",
         ]
         assert platforms == expected
 
+    def test_linux_platforms_manylinux_glibc3(self, monkeypatch):
+        # test for a future glic 3.x version
+        monkeypatch.setattr(tags, "_glibc_version_string", lambda: "3.2")
+        monkeypatch.setattr(tags, "_is_manylinux_compatible", lambda name, *args: True)
+        monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_aarch64")
+        monkeypatch.setattr(
+            sys,
+            "executable",
+            os.path.join(os.path.dirname(__file__), "hello-world-aarch64"),
+        )
+        platforms = list(tags._linux_platforms(is_32bit=False))
+        expected = (
+            ["manylinux_3_2_aarch64", "manylinux_3_1_aarch64", "manylinux_3_0_aarch64"]
+            + ["manylinux_2_{}_aarch64".format(i) for i in range(50, 16, -1)]
+            + ["manylinux2014_aarch64", "linux_aarch64"]
+        )
+        assert platforms == expected
+
     def test_linux_platforms_manylinux2014_armv6l(self, monkeypatch):
         monkeypatch.setattr(
             tags, "_is_manylinux_compatible", lambda name, _: name == "manylinux2014"
         )
         monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_armv6l")
+        monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.20", raising=False)
         platforms = list(tags._linux_platforms(is_32bit=True))
         expected = ["linux_armv6l"]
         assert platforms == expected
@@ -532,7 +639,7 @@ class TestManylinuxPlatform:
     def test_linux_platforms_not_manylinux_abi(
         self, monkeypatch, machine, abi, alt_machine
     ):
-        monkeypatch.setattr(tags, "_is_manylinux_compatible", lambda name, _: True)
+        monkeypatch.setattr(tags, "_is_manylinux_compatible", lambda name, _: False)
         monkeypatch.setattr(
             distutils.util, "get_platform", lambda: "linux_{}".format(machine)
         )
@@ -715,20 +822,20 @@ class TestCPythonTags:
 
     def test_all_args(self):
         result_iterator = tags.cpython_tags(
-            (3, 11), ["cp3_11d", "cp3_11"], ["plat1", "plat2"]
+            (3, 11), ["cp311d", "cp311"], ["plat1", "plat2"]
         )
         result = list(result_iterator)
         assert result == [
-            tags.Tag("cp3_11", "cp3_11d", "plat1"),
-            tags.Tag("cp3_11", "cp3_11d", "plat2"),
-            tags.Tag("cp3_11", "cp3_11", "plat1"),
-            tags.Tag("cp3_11", "cp3_11", "plat2"),
-            tags.Tag("cp3_11", "abi3", "plat1"),
-            tags.Tag("cp3_11", "abi3", "plat2"),
-            tags.Tag("cp3_11", "none", "plat1"),
-            tags.Tag("cp3_11", "none", "plat2"),
-            tags.Tag("cp3_10", "abi3", "plat1"),
-            tags.Tag("cp3_10", "abi3", "plat2"),
+            tags.Tag("cp311", "cp311d", "plat1"),
+            tags.Tag("cp311", "cp311d", "plat2"),
+            tags.Tag("cp311", "cp311", "plat1"),
+            tags.Tag("cp311", "cp311", "plat2"),
+            tags.Tag("cp311", "abi3", "plat1"),
+            tags.Tag("cp311", "abi3", "plat2"),
+            tags.Tag("cp311", "none", "plat1"),
+            tags.Tag("cp311", "none", "plat2"),
+            tags.Tag("cp310", "abi3", "plat1"),
+            tags.Tag("cp310", "abi3", "plat2"),
             tags.Tag("cp39", "abi3", "plat1"),
             tags.Tag("cp39", "abi3", "plat2"),
             tags.Tag("cp38", "abi3", "plat1"),
@@ -798,11 +905,11 @@ class TestCPythonTags:
         assert tags.Tag("cp38", "none", "any") in result
 
     def test_abi_defaults_needs_underscore(self, monkeypatch):
-        monkeypatch.setattr(tags, "_cpython_abis", lambda _1, _2: ["cp3_11"])
+        monkeypatch.setattr(tags, "_cpython_abis", lambda _1, _2: ["cp311"])
         result = list(tags.cpython_tags((3, 11), platforms=["any"]))
-        assert tags.Tag("cp3_11", "cp3_11", "any") in result
-        assert tags.Tag("cp3_11", "abi3", "any") in result
-        assert tags.Tag("cp3_11", "none", "any") in result
+        assert tags.Tag("cp311", "cp311", "any") in result
+        assert tags.Tag("cp311", "abi3", "any") in result
+        assert tags.Tag("cp311", "none", "any") in result
 
     def test_platforms_defaults(self, monkeypatch):
         monkeypatch.setattr(tags, "_platform_tags", lambda: ["plat1"])
@@ -812,7 +919,7 @@ class TestCPythonTags:
     def test_platforms_defaults_needs_underscore(self, monkeypatch):
         monkeypatch.setattr(tags, "_platform_tags", lambda: ["plat1"])
         result = list(tags.cpython_tags((3, 11), abis=["whatever"]))
-        assert tags.Tag("cp3_11", "whatever", "plat1") in result
+        assert tags.Tag("cp311", "whatever", "plat1") in result
 
     def test_major_only_python_version(self):
         result = list(tags.cpython_tags((3,), ["abi"], ["plat"]))
@@ -950,14 +1057,14 @@ class TestCompatibleTags:
         ]
 
     def test_all_args_needs_underscore(self):
-        result = list(tags.compatible_tags((3, 11), "cp3_11", ["plat1", "plat2"]))
+        result = list(tags.compatible_tags((3, 11), "cp311", ["plat1", "plat2"]))
         assert result == [
-            tags.Tag("py3_11", "none", "plat1"),
-            tags.Tag("py3_11", "none", "plat2"),
+            tags.Tag("py311", "none", "plat1"),
+            tags.Tag("py311", "none", "plat2"),
             tags.Tag("py3", "none", "plat1"),
             tags.Tag("py3", "none", "plat2"),
-            tags.Tag("py3_10", "none", "plat1"),
-            tags.Tag("py3_10", "none", "plat2"),
+            tags.Tag("py310", "none", "plat1"),
+            tags.Tag("py310", "none", "plat2"),
             tags.Tag("py39", "none", "plat1"),
             tags.Tag("py39", "none", "plat2"),
             tags.Tag("py38", "none", "plat1"),
@@ -978,10 +1085,10 @@ class TestCompatibleTags:
             tags.Tag("py31", "none", "plat2"),
             tags.Tag("py30", "none", "plat1"),
             tags.Tag("py30", "none", "plat2"),
-            tags.Tag("cp3_11", "none", "any"),
-            tags.Tag("py3_11", "none", "any"),
+            tags.Tag("cp311", "none", "any"),
+            tags.Tag("py311", "none", "any"),
             tags.Tag("py3", "none", "any"),
-            tags.Tag("py3_10", "none", "any"),
+            tags.Tag("py310", "none", "any"),
             tags.Tag("py39", "none", "any"),
             tags.Tag("py38", "none", "any"),
             tags.Tag("py37", "none", "any"),
@@ -1017,11 +1124,11 @@ class TestCompatibleTags:
 
     def test_default_python_version_needs_underscore(self, monkeypatch):
         monkeypatch.setattr(sys, "version_info", (3, 11))
-        result = list(tags.compatible_tags(interpreter="cp3_11", platforms=["plat"]))
+        result = list(tags.compatible_tags(interpreter="cp311", platforms=["plat"]))
         assert result == [
-            tags.Tag("py3_11", "none", "plat"),
+            tags.Tag("py311", "none", "plat"),
             tags.Tag("py3", "none", "plat"),
-            tags.Tag("py3_10", "none", "plat"),
+            tags.Tag("py310", "none", "plat"),
             tags.Tag("py39", "none", "plat"),
             tags.Tag("py38", "none", "plat"),
             tags.Tag("py37", "none", "plat"),
@@ -1032,10 +1139,10 @@ class TestCompatibleTags:
             tags.Tag("py32", "none", "plat"),
             tags.Tag("py31", "none", "plat"),
             tags.Tag("py30", "none", "plat"),
-            tags.Tag("cp3_11", "none", "any"),
-            tags.Tag("py3_11", "none", "any"),
+            tags.Tag("cp311", "none", "any"),
+            tags.Tag("py311", "none", "any"),
             tags.Tag("py3", "none", "any"),
-            tags.Tag("py3_10", "none", "any"),
+            tags.Tag("py310", "none", "any"),
             tags.Tag("py39", "none", "any"),
             tags.Tag("py38", "none", "any"),
             tags.Tag("py37", "none", "any"),
@@ -1077,6 +1184,10 @@ class TestCompatibleTags:
 
 
 class TestSysTags:
+    def teardown_method(self):
+        # Clear the version cache
+        tags._glibc_version = []
+
     @pytest.mark.parametrize(
         "name,expected",
         [("CPython", "cp"), ("PyPy", "pp"), ("Jython", "jy"), ("IronPython", "ip")],
@@ -1149,3 +1260,103 @@ class TestSysTags:
             "py" + tags._version_nodot((sys.version_info[0], 0)), "none", "any"
         )
         assert result[-1] == expected
+
+    def test_linux_platforms_manylinux2014_armv6l(self, monkeypatch, manylinux_module):
+        monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_armv6l")
+        monkeypatch.setattr(os, "confstr", lambda x: "glibc 2.20", raising=False)
+        platforms = list(tags._linux_platforms(is_32bit=True))
+        expected = ["linux_armv6l"]
+        assert platforms == expected
+
+    def test_skip_manylinux_2014(self, monkeypatch, manylinux_module):
+        monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_ppc64")
+        monkeypatch.setattr(tags, "_get_glibc_version", lambda: (2, 20))
+        monkeypatch.setattr(
+            manylinux_module, "manylinux2014_compatible", False, raising=False
+        )
+        expected = [
+            "manylinux_2_20_ppc64",
+            "manylinux_2_19_ppc64",
+            "manylinux_2_18_ppc64",
+            # "manylinux2014_ppc64",  # this one is skipped
+            # "manylinux_2_17_ppc64", # this one is also skipped
+            "linux_ppc64",
+        ]
+        platforms = list(tags._linux_platforms())
+        assert platforms == expected
+
+    @pytest.mark.parametrize(
+        "machine, abi, alt_machine",
+        [("x86_64", "x32", "i686"), ("armv7l", "armel", "armv7l")],
+    )
+    def test_linux_platforms_not_manylinux_abi(
+        self, monkeypatch, manylinux_module, machine, abi, alt_machine
+    ):
+        monkeypatch.setattr(
+            distutils.util, "get_platform", lambda: "linux_{}".format(machine)
+        )
+        monkeypatch.setattr(
+            sys,
+            "executable",
+            os.path.join(
+                os.path.dirname(__file__), "hello-world-{}-{}".format(machine, abi)
+            ),
+        )
+        platforms = list(tags._linux_platforms(is_32bit=True))
+        expected = ["linux_{}".format(alt_machine)]
+        assert platforms == expected
+
+    @pytest.mark.parametrize(
+        "machine, major, minor, tf", [("x86_64", 2, 20, False), ("s390x", 2, 22, True)]
+    )
+    def test_linux_use_manylinux_compatible(
+        self, monkeypatch, manylinux_module, machine, major, minor, tf
+    ):
+        def manylinux_compatible(tag_major, tag_minor, tag_arch):
+            if tag_major == 2 and tag_minor == 22:
+                return tag_arch == "s390x"
+            return False
+
+        monkeypatch.setattr(tags, "_get_glibc_version", lambda: (major, minor))
+        monkeypatch.setattr(
+            distutils.util, "get_platform", lambda: "linux_{}".format(machine)
+        )
+        monkeypatch.setattr(
+            manylinux_module,
+            "manylinux_compatible",
+            manylinux_compatible,
+            raising=False,
+        )
+        platforms = list(tags._linux_platforms(is_32bit=False))
+        if tf:
+            expected = ["manylinux_2_22_{}".format(machine)]
+        else:
+            expected = []
+        expected.append("linux_{}".format(machine))
+        assert platforms == expected
+
+    def test_linux_use_manylinux_compatible_none(self, monkeypatch, manylinux_module):
+        def manylinux_compatible(tag_major, tag_minor, tag_arch):
+            if tag_major == 2 and tag_minor < 25:
+                return False
+            return None
+
+        monkeypatch.setattr(tags, "_get_glibc_version", lambda: (2, 30))
+        monkeypatch.setattr(distutils.util, "get_platform", lambda: "linux_x86_64")
+        monkeypatch.setattr(
+            manylinux_module,
+            "manylinux_compatible",
+            manylinux_compatible,
+            raising=False,
+        )
+        platforms = list(tags._linux_platforms(is_32bit=False))
+        expected = [
+            "manylinux_2_30_x86_64",
+            "manylinux_2_29_x86_64",
+            "manylinux_2_28_x86_64",
+            "manylinux_2_27_x86_64",
+            "manylinux_2_26_x86_64",
+            "manylinux_2_25_x86_64",
+            "linux_x86_64",
+        ]
+        assert platforms == expected
